###############################################################################
###############################################################################
#####                            Ventana.tcl
###############################################################################
##### Creates the window which will provide some information to the user 
##### while a file is being downloaded.
###############################################################################
##### Copyright 1999-2001 Andrs Garca Garca - fandom@retemail.es
##### Distributed under the terms of the GPL v2
###############################################################################

namespace eval Ventana {

source [file join "$dirGetleft(scripts)" Rizo.tcl]

###############################################################################
# SetButtonsWidth
#    Calculates the width the buttons in the downloading window should have
#    in order to show the complete text.
#
# Returns
#    The width we need.
###############################################################################
proc SetButtonsWidth {} {
    global   labelButtons

    set widthPause  [string length $labelButtons(pause)]
    set widthSkip   [string length $labelButtons(skip)]
    set widthResume [string length $labelButtons(resume)]

    set buttonWidth [expr {$widthPause >$widthSkip}   ? $widthPause  : $widthSkip]
    set buttonWidth [expr {$buttonWidth>$widthResume} ? $buttonWidth : $widthResume]
    set buttonWidth [expr $buttonWidth -1]
    set buttonWidth [expr {$buttonWidth>8}            ? $buttonWidth : 8]

    return $buttonWidth
}

###############################################################################
# PauseDownloading
#    Pauses the downloading according to the menu entries.
#
# Parameter
#    which: either 'file' or 'page'
###############################################################################
proc Pause {which} {
    global getleftOptions labelTitles labelMessages

    set windowState [wm state .bajando]
    if {$windowState=="iconic"} {
        wm deiconify .bajando
    }
    wm withdraw .bajando
    tk_messageBox -title $labelTitles(paused) -icon info -type ok \
         -message $labelMessages(paused)
    if {$which=="file"} {
        tkwait variable getleftOptions(pauseFile)
    } else {
        tkwait variable getleftOptions(pausePage)
    }
    wm deiconify .bajando
    if {$windowState=="iconic"} {
        wm iconify .bajando
    }

    return
}

###############################################################################
# CancelDownloading
#    This procedure is invoked from the downloading window when you click on
#    the 'cancel' button or you want to close the window
#
# Parameter:
#    parent: The window over which the asking dialog will appear.
#    skip: if skip is 'skip' the current file will be skipped and the next one
#    will begin to be downloaded. If there is no parameter, the downloading
#    will be stopped.
#
# Returns
#    '0' if we are not going to stop.
#    '1' if we are.
###############################################################################
proc CancelDownloading {parent {skip {} }} {
    global labelTitles labelMessages
    global getleftState getleftOptions
    global ::Ventana::Rizo::curlReport
    variable afterId
    variable windowPid
    variable window   

    if {$getleftState(delayedDown)==0} {
        set what [tk_messageBox -type yesno -icon question -parent $parent \
                -title $labelTitles(confirm)                               \
                -message $labelMessages(stopDownload)]
        if {$what=="no"} {
            return 0
        }
    }
    catch {after cancel $afterId}
    catch {Rizo::Lector $::Ventana::Rizo::pipe stopNow}

    set getleftOptions(cancelDown)     1
    set Ventana::Rizo::curlReport(end) 1

    if {$getleftState(autoDown)==1} {
        mainWin::SetAutoDownload
    }
    if {($skip=="")||($getleftState(filesChosen)==0)} {
        set getleftOptions(stopFile)  1
        set getleftState(downloading) 0
        if {$getleftState(waiting)==1} {
            after cancel waitingPid
            set getleftState(waiting) 0
        }
        catch {wm withdraw $window(top)}
    }

    incr curlReport(nextFile)

    return 1
}

###############################################################################
# TrimString
#    Trims the given string so that if it is longer than the available space
#    the middle of it gets substituted with '...', and puts the resulting
#    string in the given label.
#
# Parmetros
#    cadena: string to trim.
#    labelPath: the label in which the string is going to be shown
###############################################################################
proc TrimString {cadena labelPath} {

    set font     [$labelPath cget -font]
    set color    [$labelPath cget -bg]
    set tmpColor [$labelPath cget -fg]
    set varName  [$labelPath cget -textvariable]

    upvar #0 $varName textVariable
    $labelPath configure -fg $color

    set textVariable $cadena
    set labelLength  [expr {[winfo width $labelPath]-5}]
    set strLength    [font measure $font -displayof $labelPath $cadena]

    if {$labelLength<=$strLength} {
        set strChars [string length $cadena]
        set middle   [expr {round($strChars/2)-2}]
        set primer   [string range $cadena 0 $middle]
        set segun    [string range $cadena [expr {$middle+1}] end]
        set textVariable ""
        append textVariable $primer "..." $segun
        while {[font measure $font -displayof $labelPath $textVariable]>$labelLength} {
            regsub {(.)(\.\.\.)(.)} $textVariable {...} textVariable
        }
        BalloonHelp::set_balloon $labelPath $cadena
    } else {
        BalloonHelp::delete_balloon $labelPath
    }

    $labelPath configure -fg $tmpColor

    return
}

###############################################################################
# FileStrings
#    Controls the strings that will be put in the downloading window, to
#    show the name and directory of the file being processed or downloaded
#
# Parameter:
#    url to be downloaded or preprocessed
###############################################################################
proc FileStrings {url} {
    global directories labelMessages
    variable window

    set fileName  [::Commands::UrlToFile $url]
    set parsedUrl [HtmlParser::ParseUrl $url]
    set domain    [string tolower [lindex $parsedUrl 1]]
    regsub -all {:}  $domain {} domain
    regexp "(?:$directories(base)/$domain/)(.*)" $fileName nada tmp
    if {![regexp {(.*)(?:/)(.*)} $tmp nada tmpDir tmpFile]} {
        set tmpDir ""
        set tmpFile $tmp
    }

    TrimString $domain  $window(url)
    TrimString $tmpDir  $window(dir)
    TrimString $tmpFile $window(file)

    update

    return
}

###############################################################################
# DownloadWindow
#    Creates the window, in which some info about the downloading will be
#    displayed.
###############################################################################
proc DownloadWindow {} {
    global labelButtons labelDialogs
    global tcl_platform dirGetleft
    variable window

    set win [toplevel .bajando]
    wm geometry  $win 290x160
    wm resizable $win 0 0

    set marco     [frame $win.marco]
    set marco_in  [frame $marco.marco_interno]
    set marco_url [frame $marco_in.url]
    set marco_dir [frame $marco_in.dir]
    set marco_fil [frame $marco_in.fil]
    set marco_que [frame $marco_in.que]
    set marco_bar [frame $marco_in.bar]
    set botones   [frame $win.botones]

    set imaUrl    [image create photo \
            -file [file join "$dirGetleft(images)" html.gif]]
    set imaFolder [image create photo \
            -file [file join "$dirGetleft(images)" folder.gif]]
    set imaFiles  [image create photo \
            -file [file join "$dirGetleft(images)" files.gif]]
    set imaClock  [image create photo \
            -file [file join "$dirGetleft(images)" clock.gif]]

    set etq_url   [label $marco_url.etq_url  -image $imaUrl]
    set etq_dir   [label $marco_dir.etq_dir  -image $imaFolder]
    set etq_file  [label $marco_fil.etq_file -image $imaFiles]
    set etq_que   [label $marco_que.etq_que  -image $imaClock]

    BalloonHelp::set_balloon $etq_url  $labelDialogs(url)
    BalloonHelp::set_balloon $etq_dir  $labelDialogs(dir)
    BalloonHelp::set_balloon $etq_file $labelDialogs(file)
    BalloonHelp::set_balloon $etq_que  $labelDialogs(left)

    set etq_url2  [label $marco_url.etq_url2  -width 45 -anchor w \
            -bd 2 -relief sunken -textvariable Ventana::window(labelUrl)]
    set etq_dir2  [label $marco_dir.etq_dir2  -width 45 -anchor w \
            -textvariable Ventana::window(labelDir)]
    set etq_file2 [label $marco_fil.etq_file2 -width 45 -anchor w \
            -textvariable Ventana::window(labelFile)]
    set etq_que2  [label $marco_que.etq_que2  -width 30 -anchor w \
            -textvariable Ventana::Rizo::curlReport(left)]

    set progressBar [Progreso::ProgressBar $marco_bar.progress]

    set window(top)  $win
    set window(url)  $etq_url2
    set window(dir)  $etq_dir2
    set window(file) $etq_file2
    set window(left) $etq_que2
    set window(bar)  $progressBar

    set buttonWidth [SetButtonsWidth]
    set pause    [button $botones.pause    -textvariable labelButtons(pause)\
        -command ::Ventana::PauseDownloading  -width $buttonWidth]
    set cancelar [button $botones.cancelar -textvariable labelButtons(skip) \
        -command "::Ventana::CancelDownloading $Ventana::window(top) skip"  \
                  -width $buttonWidth]
    set window(cancel) $cancelar
    set window(pause)  $pause

    pack $marco     -ipady 5
    pack $botones   -fill x -padx 7
    pack $marco_in  -padx 10 -side bottom
    pack $marco_url $marco_in $marco_dir $marco_fil $marco_que $marco_bar \
            -fill x
    pack $etq_url   $etq_url2  -side left -pady 2 -padx 2
    pack $etq_dir   $etq_dir2  -side left -pady 2 -padx 2
    pack $etq_file  $etq_file2 -side left -pady 2 -padx 2
    pack $etq_que   $etq_que2  -side left -pady 2 -padx 2
    pack $progressBar
    pack $cancelar $pause -side right -padx 3 -pady 5

    wm protocol $win WM_DELETE_WINDOW "::Ventana::CancelDownloading $win"

    update

    return
}

###############################################################################
# ShowWindow
#   If needed, maps the downloading window, and sets its title bar
#
# Parameter
#   which: either 'head', 'down', 'process' or 'wait' depending on the action 
#          taking place.
###############################################################################
proc ShowWindow {which} {
    global   labelTitles labelButtons getleftState
    variable window

    if {$::getleftState(downloading)==0} {
        if {$::DEBUG==1} {
            tk_messageBox -title Error -icon error -type ok \
                    -message "Pringamos en ShowWindow: $which"
        }
        return
    }   

    if {![winfo exists .bajando]} {
        DownloadWindow
    } else {
        set state [wm state .bajando]
        if {$state=="withdrawn"} {
            wm deiconify .bajando
        }
    }
    switch -exact $which {
        head {
            wm title .bajando $labelTitles(connect)
            $window(cancel) configure -state normal
        }
        down {
            wm title .bajando $labelTitles(download)
            $window(cancel) configure -state normal
        }
        process {
            wm title .bajando $labelTitles(process)
            $window(cancel) configure -state disabled
        }
        wait {
            StartWaiting 10
        }
    }
    return
}

###############################################################################
# FileDownloading
#   Creates a nice window to show info about how the downloading goes
#
# Parameters:
#    fichero: file in which the link will be saved.
#    enlace:  file url.
#    madre:   referer page for the link
###############################################################################
proc FileDownloading {fichero enlace madre} {
    global   getleftState directories
    global   labelTitles
    global   Ventana::Rizo::meta
    global   Ventana::Rizo::curlReport
    variable window
    variable file
    variable action

    if {$getleftState(downloading)==0} {
        return
    }

    set file   $fichero
    set action body

    ShowWindow down
    FileStrings $enlace

    set curlReport(end)   0
    set curlReport(long)  1
    set curlReport(pause) 0
    set curlReport(left) ""

    if {([file exists $fichero])&&($meta(versionServer)>=1.1)} {
        set size [file size $fichero]
        if {($size>$meta(totalBytes))&&($meta(totalBytes)!=0)} {
            file delete $fichero
        } elseif {($size==$meta(totalBytes))&&($meta(totalBytes)!=0)} {
            after 1000 "set ::Ventana::Rizo::curlReport(nextFile) 1"
            return
        }
        Ventana::Rizo::DataRequest $fichero $enlace $madre 1
    } else {
        Ventana::Rizo::DataRequest $fichero $enlace $madre
    }
    BarUpdate

    return
}

###############################################################################
# HeadDownloading
#    Gets the headers of the url from the server.
#
# Parameters:
#    enlace: url to download
#    madre:  referer page of the link
###############################################################################
proc HeadDownloading {enlace madre} {
    global siteUrl getleftState
    variable link
    variable mother
    variable action

    set link   $enlace
    set mother $madre
    set action head

    ShowWindow head
    FileStrings $enlace

    set Ventana::Rizo::curlReport(end) 0
    set ::Ventana::Rizo::curlReport(left) ""
    Ventana::Rizo::HeadRequest $enlace $madre
    tkwait variable Ventana::Rizo::curlReport(end)
           
    return
}

###############################################################################
# PauseDownloading
#    Takes control when you click on the 'Pause' button
###############################################################################
proc PauseDownloading {} {
    global labelButtons
    variable window

    set Ventana::Rizo::curlReport(pause) 1
    $window(pause) configure -textvariable labelButtons(resume) \
            -command ::Ventana::ResumeDownloading

    return
}

###############################################################################
# ResumeDownloading
#    Takes control when you click on the 'Resume' button
###############################################################################
proc ResumeDownloading {} {
    global labelButtons
    variable window
    variable file
    variable link
    variable mother
    variable action

    set Ventana::Rizo::curlReport(pause) 0
    $window(pause) configure -textvariable labelButtons(pause) \
                -command ::Ventana::PauseDownloading

    if {$action=="body"} {
        Progreso::ProgressBarReset $window(bar)
        Ventana::Rizo::DataRequest "$file" "$link" "$mother" 1
        Ventana::BarUpdate
    } else {
        HeadDownloading "$link" "$mother"
    }
    return
}

###############################################################################
# StartWaiting
#    This procedure is invoked when a timeout has ocurred, to wait before
#    trying again.
#
# Parameter
#    seconds: The seconds yet to wait
###############################################################################
proc StartWaiting {seconds} {
    global   getleftState labelTitles
    variable window
    variable waitingPid

    if {$seconds==0} {
        set getleftState(waiting) 0
        return
    }

    wm title .bajando "$labelTitles(waiting) $seconds"
    $window(pause) configure -command ::Ventana::InterruptWaiting \
            -textvariable labelButtons(resume)

    set waitingPid [after 1000 ::Ventana::StartWaiting [expr $seconds -1]]

    return
}

###############################################################################
# InterruptWaiting
#    While waiting to retry after a cancelled download, if you click on resume
#    this procedure will be invoked.
###############################################################################
proc InterruptWaiting {} {
    global   getleftState
    variable window
    variable waitingPid

    after cancel $waitingPid
    $window(pause) configure -command ::Ventana::ResumeDownloading
    set getleftState(waiting) 0

    return
}

###############################################################################
# BarUpdate
#    Updates the progress bar.
###############################################################################
proc BarUpdate {} {
    global Ventana::Rizo::curlReport
    global Ventana::Rizo::curlError
    global Ventana::Rizo::meta
    global labelDialogs downOptions
    variable file
    variable link
    variable mother
    variable window
    variable afterId

    if {[string match $curlReport(percentage) ""]} return
    Progreso::ProgressBarUpdate $window(bar) $curlReport(percentage)

    if {$curlReport(pause)==0} {
        if {($curlReport(end)==0) && ($curlReport(stop)!=1)} {
            set afterId [after 1000 Ventana::BarUpdate]
        } else {
            if {($curlError==18)||($curlError==28)} {
                Herramientas::ErrorLogEnter $link $mother \
                        $labelDialogs(fileIncom)
                if {($downOptions(resume)==1)&&($curlReport(stop)!=1)} {
                    Progreso::ProgressBarReset $window(bar)
                    if {$curlError==18} {
                        FileDownloading $Ventana::file \    
                                $Ventana::link $Ventana::mother
                    } else {
                        catch {incr curlReport(nextFile)}
                    }
                    return
                }
            }
            Progreso::ProgressBarFull $window(bar)
            update
            after 75
            Progreso::ProgressBarReset $window(bar)
            $window(dir)  configure -text ""
            $window(file) configure -text ""
            catch {incr curlReport(nextFile)}
        }
    }
    return
}

}
