###############################################################################
###############################################################################
##                                  Delay.tcl
###############################################################################
###############################################################################
## This namespace is used to program delayed downloads into Getleft.
###############################################################################
###############################################################################
## (c) 2001 Andrs Garca Garca. fandom@retemail.es
## You may distribute the contents of this file under the terms of the GPL v2
###############################################################################
###############################################################################

namespace eval Delay {

###############################################################################
# DelayedCancelDelay
#    Cancel the event that would start or/and the end of a delayed download.
###############################################################################
proc DelayedCancelDelay {} {
    variable delayedDownload
    global   getleftState

    catch {after cancel $delayedDownload(startEvent)}
    catch {after cancel $delayedDownload(endEvent)}

    set getleftState(delaySet)    0
    set getleftState(delayedDown) 0

    $mainWin::taskbar(delay) configure -relief flat

    return
}

###############################################################################
# DelayedStartDownload
#    Starts the download at the given time.
###############################################################################
proc DelayedStartDownload {} {
    variable delayedDownload
    variable window
    global getleftState

    if {$delayedDownload(noEnd)==0} {
        set clicksToEnd [expr ($delayedDownload(endTime)-[clock seconds])*1000]
        if {$clicksToEnd<0} {
            set clicksToEnd 0
        }
        set delayedDownload(endEvent) \
            [after $clicksToEnd Delay::DelayedEndDownload]
    }
    if {$delayedDownload(days)==1} {
        DelayedStartTime $delayedDownload(startTime)
    }
    set getleftState(delayedDown) 1
    mainWin::SetAutoDownload

    return
}

###############################################################################
# DelayedEndDownload
#    Ends the download in the given time.
###############################################################################
proc DelayedEndDownload {} {
    variable delayedDownload
    global   getleftOptions

    switch $delayedDownload(how) {
        0 {
            ::Ventana::CancelDownloading .
        }
        1 {
            set getleftOptions(stopFile) 1
            mainWin::SetAutoDownload
        }
        2 {
            set getleftOptions(stopPage) 1
            mainWin::SetAutoDownload
        }
    }
    if {$delayedDownload(days)==0} {
        $mainWin::taskbar(delay) configure -relief flat
    }

    return
}

###############################################################################
# DelayedStartTime
#    Calculates the time the next download will begin.
#
# Parameter
#    The time in seconds from 01-01-1970 when the download will start.
###############################################################################
proc DelayedStartTime {startTime} {
    variable delayedDownload

    set now [clock seconds]

    if {$delayedDownload(days)==0} {
        if {$startTime<$now} {
            set startTime [expr $startTime+3600*24]
        }
    } else {
        set hour   [clock format $startTime -format "%H"]
        set minute [clock format $startTime -format "%M"]
        set startTime 9999999999
        foreach day [list monday tuesday  wednesday thursday \
                          friday saturday sunday] {
            if {$delayedDownload($day)==1} {
                set startTmp [clock scan "$day $hour:$minute"]
                if {($startTmp<$startTime)&&($startTmp>$now)} {
                    set startTime $startTmp
                }
            }
        }
    }
    set delayedDownload(startTime) $startTime
    DelayedSetStartTime

    return
}

###############################################################################
# DelayedSetStartTime
#    Given the time when to start, this procedure will create the event to
#    start the download at that time.
###############################################################################
proc DelayedSetStartTime {} {
    variable delayedDownload

    set clicksToStart [expr ($delayedDownload(startTime)-[clock seconds])*1000]
    # This may happen if we closed Getleft and we are running it again.
    if {$clicksToStart<0} {
        DelayedStartTime $delayedDownload(startTime)
        return
    }        
    set delayedDownload(startEvent)  \
            [after $clicksToStart Delay::DelayedStartDownload]

    return 
}

###############################################################################
# DelayedEndTime
#    Calculates the time the next download will stop.
#
# Parametes
#    The hour and the minute of the time and the start time.
#
# Returns:
#    The time in seconds from 01-01-1970
###############################################################################
proc DelayedEndTime {hour minute startTime} {
    variable delayedDownload

    set endTime [clock scan "today $hour:$minute"]
    while {$startTime>=$endTime} {
        set endTime [expr $endTime + 3600*24]
    }
    return $endTime
}

###############################################################################
# DelayedCheckTime
#    Checks whether the given time makes sense or not.
#
# Parametes
#    The hour and the minute of the time to check.
#
# Returns:
#    '0' makes sense, '1' doesn't.
###############################################################################
proc DelayedCheckTime {hour minute} {

    if {(![regexp {^[0-9]+$} $hour])||(![regexp {^[0-9]+$} $minute])} {
        return 1
    }
    if {($hour<0)||($hour>23)||($minute<0)||($minute>59)} {
        return 1
    }

    return 0
}

###############################################################################
# DelayedActivateEndTime
#    Activates or deactivates the spinboxes with the time to finish downloading
###############################################################################
proc DelayedActivateEndTime {} {
    variable delayedDownloadTemp
    variable window

    if {$delayedDownloadTemp(noEnd)==1} {
        CuadroSpin::EnableSpin $window(endHour)   disable
        CuadroSpin::EnableSpin $window(endMinute) disable
    } else {
        CuadroSpin::EnableSpin $window(endHour)   normal
        CuadroSpin::EnableSpin $window(endMinute) normal
    }

    return
}

###############################################################################
# DelayedActivateDays
#    Activates or deactivates the 'days' buttons.
###############################################################################
proc DelayedActivateDays {} {
    variable delayedDownloadTemp
    variable dayButton

    if {$delayedDownloadTemp(days)==0} {
        $dayButton(monday)    configure -state disable
        $dayButton(tuesday)   configure -state disable
        $dayButton(wednesday) configure -state disable
        $dayButton(thursday)  configure -state disable
        $dayButton(friday)    configure -state disable
        $dayButton(saturday)  configure -state disable
        $dayButton(sunday)    configure -state disable
    } else {
        if {[$dayButton(monday) cget -state]!="normal"} {
            $dayButton(monday)    configure -state normal
            $dayButton(tuesday)   configure -state normal
            $dayButton(wednesday) configure -state normal
            $dayButton(thursday)  configure -state normal
            $dayButton(friday)    configure -state normal
            $dayButton(saturday)  configure -state normal
            $dayButton(sunday)    configure -state normal
        }
    }
    return
}

###############################################################################
# DelayedActivateStop
#    Activates or deactivates the 'days' buttons.
###############################################################################
proc DelayedActivateStop {} {
    variable delayedDownloadTemp
    variable stopButton

    if {$delayedDownloadTemp(noEnd)==0} {
        if {[$stopButton(inme) cget -state]!="normal"} {
            $stopButton(inme) configure -state normal
            $stopButton(file) configure -state normal
            $stopButton(page) configure -state normal
        }
    } else {
        $stopButton(inme) configure -state disable
        $stopButton(file) configure -state disable
        $stopButton(page) configure -state disable
    }
    return
}

###############################################################################
# DelayedDownloadWindow
#    Creates the dialog to define when to start a download.
###############################################################################
proc DelayedDownloadWindow {} {
    global labelDialogs labelTitles labelFrames getleftState
    variable delayedDownloadTemp
    variable dayButton
    variable stopButton
    variable window

    if {[winfo exists .delayedDown]} {
        focus .delayedDown
        return
    }

    set win [toplevel .delayedDown]
    wm title $win $labelTitles(delayedDown)
    wm resizable $win 0 0

    set coord(x) [winfo rootx .]
    set coord(y) [winfo rooty .]
    wm geometry $win +[expr {$coord(x)+100}]+[expr {$coord(y)+15}]

    set extFrame [frame $win.extFrame]
    set intFrame [frame $extFrame.intFrame]

    set timeFrame [fl::FrameLabel $intFrame.time -bd 2 -relief groove  \
            -textvariable labelFrames(time)]
    set startLabel  [label $timeFrame.startLabel -textvariable labelDialogs(begin)]
    set startHour   [CuadroSpin::CuadroSpin $timeFrame.startHour   0 0 23 3]
    set startMinute [CuadroSpin::CuadroSpin $timeFrame.startMinute 0 0 59 3 5]
    set endLabel    [label $timeFrame.endLabel -textvariable labelDialogs(end)]
    set endHour     [CuadroSpin::CuadroSpin $timeFrame.endHour     0 0 23 3]
    set endMinute   [CuadroSpin::CuadroSpin $timeFrame.endMinute   0 0 59 3 5]
    set noEnd [checkbutton $timeFrame.noEnd -textvariable labelDialogs(noLimit) \
            -variable Delay::delayedDownloadTemp(noEnd) -command "
                Delay::DelayedActivateEndTime
                Delay::DelayedActivateStop
            "]
    if {$delayedDownloadTemp(noEnd)==""} {
        set delayedDownloadTemp(noEnd) 1
    }
    if {[info exists Delay::delayedDownloadTemp(startTime)]} {
        CuadroSpin::SetValue $startHour   \
                [clock format $delayedDownloadTemp(startTime) -format "%H"]
        CuadroSpin::SetValue $startMinute \
                [clock format $delayedDownloadTemp(startTime) -format "%M"]
    }
    if {[info exists Delay::delayedDownloadTemp(endTime)]} {
        CuadroSpin::SetValue $endHour     \
                [clock format $delayedDownloadTemp(endTime) -format "%H"]
        CuadroSpin::SetValue $endMinute   \
                [clock format $delayedDownloadTemp(endTime) -format "%M"]
    }

    set window(win)         $win
    set window(startHour)   $startHour
    set window(startMinute) $startMinute
    set window(endHour)     $endHour
    set window(endMinute)   $endMinute

    DelayedActivateEndTime

    set howFrame  [fl::FrameLabel $intFrame.how  -bd 2 -relief groove  \
            -textvariable labelFrames(stop)]
    set stopButton(inme) [radiobutton $howFrame.inme \
            -textvariable labelDialogs(inme)         \
            -variable Delay::delayedDownloadTemp(how) -value 0] 
    set stopButton(file) [radiobutton $howFrame.file \
            -textvariable labelDialogs(afterFile)    \
            -variable Delay::delayedDownloadTemp(how) -value 1]
    set stopButton(page) [radiobutton $howFrame.page \
            -textvariable labelDialogs(afterPage)    \
            -variable Delay::delayedDownloadTemp(how) -value 2]
    if {$delayedDownloadTemp(how)==""} {
        set delayedDownloadTemp(how) 1
    }
    DelayedActivateStop

    set daysFrame [fl::FrameLabel $intFrame.days -bd 2 -relief groove  \
            -textvariable labelFrames(days)]
    set today [radiobutton $daysFrame.today -textvariable labelDialogs(onlyToday) \
            -variable Delay::delayedDownloadTemp(days) -value 0    \
            -command Delay::DelayedActivateDays]
    set on    [radiobutton $daysFrame.on    -textvariable labelDialogs(on)        \
            -variable Delay::delayedDownloadTemp(days) -value 1    \
            -command Delay::DelayedActivateDays]
    if {$delayedDownloadTemp(days)==""} {
        set delayedDownloadTemp(days) 0
    }

    set dayButton(monday)    [checkbutton $daysFrame.monday    \
            -textvariable labelDialogs(monday)                 \
            -variable Delay::delayedDownloadTemp(monday)]
    set dayButton(tuesday)   [checkbutton $daysFrame.tuesday   \
            -textvariable labelDialogs(tuesday)                \
            -variable Delay::delayedDownloadTemp(tuesday)]
    set dayButton(wednesday) [checkbutton $daysFrame.wednesday \
            -textvariable labelDialogs(wednesday)              \
            -variable Delay::delayedDownloadTemp(wednesday)]
    set dayButton(thursday)  [checkbutton $daysFrame.thursday  \
            -textvariable labelDialogs(thursday)               \
            -variable Delay::delayedDownloadTemp(thursday)]
    set dayButton(friday)    [checkbutton $daysFrame.friday    \
            -textvariable labelDialogs(friday)                 \
            -variable Delay::delayedDownloadTemp(friday)]
    set dayButton(saturday)  [checkbutton $daysFrame.saturday  \
            -textvariable labelDialogs(saturday)               \
            -variable Delay::delayedDownloadTemp(saturday)]
    set dayButton(sunday)    [checkbutton $daysFrame.sunday    \
            -textvariable labelDialogs(sunday)                 \
            -variable Delay::delayedDownloadTemp(sunday)]
    DelayedActivateDays

    set buttonFrame [frame $win.buttonFrame]
    set accept [button $buttonFrame.accept -textvariable labelButtons(ok)      \
        -width 8 -command "Delay::DelayedDownloadControl 1"]
    set cancel [button $buttonFrame.cancel -textvariable labelButtons(cancel)  \
        -width 8 -command "Delay::DelayedDownloadControl 0"]
    set clear  [button $buttonFrame.clear -textvariable labelButtons(clear)    \
        -width 8 -command "Delay::DelayedDownloadControl 2"]

    pack $extFrame -padx 5
    pack $intFrame 
    grid $timeFrame $howFrame     -sticky news -ipadx 7 -ipady 7 -padx 5 -pady 10
    grid $daysFrame -columnspan 2 -sticky news -ipadx 7 -ipady 7 -padx 5

    grid $startLabel $startHour $startMinute -padx 5 -pady 3 -sticky w
    grid $endLabel   $endHour   $endMinute   -padx 5 -pady 3 -sticky w
    grid $noEnd -columnspan 3 -sticky w

    grid $stopButton(inme) -sticky w
    grid $stopButton(file) -sticky w
    grid $stopButton(page) -sticky w

    grid $today $on  -sticky w
    grid $dayButton(monday)    $dayButton(tuesday)                     -sticky w
    grid $dayButton(wednesday) $dayButton(thursday) $dayButton(friday) -sticky w
    grid $dayButton(saturday)  -row 3 -column 1 -sticky w
    grid $dayButton(sunday)    -row 3 -column 2 -sticky w

    pack $buttonFrame           -padx 7 -pady 5   -side right
    pack $cancel $accept $clear -padx 3 -side right

    bind $win <Escape> "$cancel invoke"

    if {$getleftState(delaySet)==0} {
        $clear configure -state disable
    } else {
        $clear configure -state normal
    }

    return
}

###############################################################################
# DelayedDownloadControl
#    Checks that the data entered in the dialog is consistent.
#
# Parameter
#    '1' if the user accept the data, '0' if the dialog was cancelled.
###############################################################################
proc DelayedDownloadControl {done} {
    global labelTitles labelMessages getleftState
    variable window
    variable delayedDownload
    variable delayedDownloadTemp

    if {$done==1} {
        set startHour   [CuadroSpin::GetValue $window(startHour)]
        set startMinute [CuadroSpin::GetValue $window(startMinute)]

        if {[DelayedCheckTime $startHour $startMinute]==1} {
            tk_messageBox -title $labelTitles(error) -parent $window(win) \
                    -message $labelMessages(wrongStart)     -type ok 
            return
        }
        if {$delayedDownloadTemp(noEnd)==0} {
            set endHour   [CuadroSpin::GetValue $window(endHour)]
            set endMinute [CuadroSpin::GetValue $window(endMinute)]

            if {[DelayedCheckTime $endHour $endMinute]==1} {
                tk_messageBox -title $labelTitles(error) -parent $window(win) \
                        -message $labelMessages(wrongStart) -type ok 
                return
            }
        }
        if {$delayedDownloadTemp(days)==1} {
            if {($delayedDownloadTemp(monday)==0)      &&($delayedDownloadTemp(tuesday)==0) \
                 &&($delayedDownloadTemp(wednesday)==0)&&($delayedDownloadTemp(thursday)==0)\
                 &&($delayedDownloadTemp(friday)==0)   &&($delayedDownloadTemp(saturday)==0)\
                 &&($delayedDownloadTemp(sunday)==0)} {
                tk_messageBox -title $labelTitles(error)   -parent $window(win)   \
                        -message $labelMessages(whichDays) -type ok 
                return
            }
        }
    }

    destroy $window(win)

    if {$done==0} return

    if {$done==2} {
        DelayedCancelDelay
        return
    }
    DelayedCancelDelay
    array set delayedDownload [array get delayedDownloadTemp]

    DelayedStartTime [clock scan "today $startHour:$startMinute"]
    if {$delayedDownload(noEnd)==0} {
        set delayedDownload(endTime) \
                [DelayedEndTime $endHour $endMinute $delayedDownload(startTime)]
    }

    $mainWin::taskbar(delay) configure -relief sunken
    set getleftState(delaySet) 1

    return
}

###############################################################################
# DelayedDownload
#    Takes care of all the trivia of defining a delayed download.
###############################################################################
proc DelayedDownload {} {
    variable delayedDownloadTemp
    variable delayedDownload

    catch {unset delayedDownloadTemp}
    array set delayedDownloadTemp [array get delayedDownload]

    DelayedDownloadWindow

    return
}

}
