###############################################################################
###############################################################################
##                             getLog.tcl
###############################################################################
###############################################################################
## Defines the Getleft's log window.
###############################################################################
###############################################################################
## (c) 2001 Andrs Garca Garca. fandom@retemail.es
## You may distribute the contents of this file under the terms of the GPL v2
###############################################################################
###############################################################################

namespace eval getLog {

################################################################################
# DefineTaskbarButtons
#    Defines the buttons in the taskbar.
#
# Parameters
#    The frame for the buttons.
################################################################################
proc DefineTaskbarButtons {frame} {
    global   labelDialogs labelMenus
    variable taskbar
    variable window

    set taskbar(start)  [button $frame.start   -image $iconNS::iconImages(start) \
            -relief flat -bd 1 -command {
                getLog::UrlLogDownload [getLog::UrlLogSelectedUrl]
            }]
    ::BalloonHelp::set_balloon $taskbar(start)  $labelDialogs(start)

    set taskbar(update) [button $frame.update -image $iconNS::iconImages(update) \
            -relief flat -bd 1 -command {
                getLog::UrlLogUpdate [getLog::UrlLogSelectedUrl]
            }]
    ::BalloonHelp::set_balloon $taskbar(update) $labelDialogs(update)

    set taskbar(queue)  [button $frame.queue -image $iconNS::iconImages(queue)   \
            -relief flat -bd 1 -command {
                getLog::UrlLogQueue [getLog::UrlLogSelectedUrl]    
            }]
    ::BalloonHelp::set_balloon $taskbar(queue)  $labelDialogs(queue)

    set taskbar(conf)  [button $frame.conf  -image $iconNS::iconImages(conf) \
            -relief flat -bd 1 -command {
                getLog::UrlLogChangeOptions [getLog::UrlLogSelectedUrl]    
            }]
    ::BalloonHelp::set_balloon $taskbar(conf)   $labelDialogs(options)

    set taskbar(exit) [button $frame.exit  -image $iconNS::iconImages(exit) \
            -relief flat -bd 1 -command "destroy $window(toplevel)"]
    ::BalloonHelp::set_balloon $taskbar(exit)   $labelDialogs(exit)

    foreach {name path} [array get taskbar] {
        bindtags $path "[bindtags $path] logTaskIcon"
    }
    bind logTaskIcon <Enter> {
        %W configure -relief raised
    }
    bind logTaskIcon <Leave> {
        %W configure -relief flat
    }

    pack $taskbar(start) $taskbar(update) $taskbar(queue) $taskbar(conf) \
            -side left
    pack $taskbar(exit) -side right

    return
}

################################################################################
# ActivateTaskbarButtons
#    Activates or disables the buttons in the taskbar depending on whether
#    there a row selected.
################################################################################
proc ActivateTaskbarButtons {} {
    variable taskbar
    variable window

    set anchor [$window(listbox) curselection]
    if {$anchor==""} {
        $taskbar(start)  configure -state disabled
        $taskbar(update) configure -state disabled
        $taskbar(queue)  configure -state disabled
        $taskbar(conf)   configure -state disabled
    } else {
        $taskbar(start)  configure -state active
        $taskbar(update) configure -state active
        $taskbar(queue)  configure -state active
        $taskbar(conf)   configure -state active
    }
    return
}

################################################################################
# UrlLogWrite
#    Reads the file with the urls in the log, hte file will be saved in the
#    dirGetleft(conf) directory with the name getLog.log.
################################################################################
proc UrlLogWrite {} {
    variable urlLog
    global dirGetleft tcl_platform


    set urlLogFile [file join $dirGetleft(conf) getLog.log]
    if {(![info exists urlLog(urlList)])||([llength $urlLog(urlList)]==0)} {
        catch {file delete $urlLogFile}
        return
    }

    set handle [open "$urlLogFile" w]

    puts $handle "# Log of the sites downloaded by Getleft."
    puts $handle "# This file is generated automatically, please do not edit."
    puts $handle "\n\n\n"

    puts $handle "set urlLog(urlList) [list $urlLog(urlList)]"
    puts $handle "\n\n"
    foreach url $urlLog(urlList) {
        puts $handle "set urlLog($url,localDir) \"$urlLog($url,localDir)\""
        puts $handle "set urlLog($url,date)     $urlLog($url,date)"
        puts $handle "set urlLog($url,options)  [list $urlLog($url,options)]"
        puts $handle "set urlLog($url,keep)     $urlLog($url,keep)"
#       puts $handle "set urlLog($url,encoding) $urlLog($url,encoding)"
    }
    puts $handle "\n\n"
    close $handle

    if {$tcl_platform(platform)=="windows"} {
        file attributes $urlLogFile -hidden 1
    } else {
        file attributes $urlLogFile -permissions 0600
    }

    return
}

################################################################################
# UrlLogRead
#    Reads the file with the urls in the log, hte file will be saved in the
#    dirGetleft(conf) directory with the name getLog.log.
#
# Side effect
#    The namespace variable urlLog will be set in all its glory.
################################################################################
proc UrlLogRead {} {
    variable urlLog
    global dirGetleft

    set urlLogFile [file join $dirGetleft(conf) getLog.log]
    if {[file exists $urlLogFile]} {
        source $urlLogFile
    } else {
        set urlLog(urlList) ""
    }

    return
}

################################################################################
# UrlLogGetIndex
#   When the user clicks in the url list, this procedure takes care of selecting
#   the nearest entry and returns its index.
#
# Parameters
#   win: window where you clicked.
#   y: the coordinate where you did it
#
# Returns
#   The index of the url, or -1 if the is none to return
################################################################################
proc UrlLogGetIndex {win y} {
    
    set ::tablelist::win [winfo parent $win]
    set ::tablelist::y [expr "$y + [winfo y $win]"]
    if {[winfo exists $::tablelist::win]} {
        set index [$::tablelist::win nearest $::tablelist::y]
	    if {$index!=-1} {
            if {[catch {tkListboxBeginSelect $::tablelist::win $index}]} {
                tk::ListboxBeginSelect $::tablelist::win $index
            }
	    }
	    return $index
    }
    return -1
}

################################################################################
# UrlLogUrlIndex
#   Given an Url this procedure returns its index in the listbox.
#
# Parameters
#   url: The url to search for.
#
# Returns
#   The index of the url, or -1 if the is none to return
################################################################################
proc UrlLogUrlIndex {url} {
    variable window
    variable urlLog

    if {(![info exists window(listbox)])||(![winfo exists $window(listbox)])} {
        return -1
    }

    set rowNumber [$window(listbox) size]
    for {set i 0} {$i<$rowNumber} {incr i} {
        set site [$window(listbox) cellcget $i,0 -text]
        if {[string match $url $site]} {
            return $i
        }
    }

    return -1
}

################################################################################
# UrlLogEnter
#    Enters one Url and its data into the urlLog. This procedure is invoked
#    after the site has been downloaded.
#
# Parameters
#    url: The url to enter.
#    dir: The directory where it has been saved.
#    options: A list with the options for downloading the site.
#
# Side effect: To begin with the keep bit will be set to '0'.
################################################################################
proc UrlLogEnter {url dir options} {
    variable urlLog
    variable window

    set oldIndex [lsearch $urlLog(urlList) $url]
    if {$oldIndex==-1} {
        set urlNumber [llength $urlLog(urlList)]
        if {$urlNumber>=100} {
            for {set i 0} {$i<$urlNumber} {incr i} {
                set urlTmp [lindex $urlLog(urlList) $i]
                if {$urlLog($urlTmp,keep)==0} {
                    UrlLogRemove $urlTmp
                    break
                }
            }
            if {$i==$urlNumber} {
                return
            }
        }
        lappend urlLog(urlList) $url
    }

    set urlLog($url,localDir) "$dir"
    set urlLog($url,date)     [clock seconds]
    set urlLog($url,options)  $options
    if {![info exists urlLog($url,keep)]} {
        set urlLog($url,keep) 0
    }

    if {$oldIndex!=1} {
        if {([info exists window])&&([winfo exists $window(listbox)])} {
            $window(listbox) delete $oldIndex
            GetLogWindowRow $url $dir
        }
    }

    return
}

################################################################################
# UrlLogSelectedUrl
#   When the user clicks on the icons, this procedure is used to know the url
#   of the row that was selected at the time.
#
# Returns
#   The url or an empty string if no row is selected
################################################################################
proc UrlLogSelectedUrl {} {
    variable window

    set anchor [$window(listbox) curselection]
    if {[catch {$window(listbox) cellcget $anchor,0 -text} url]} {
        return
    }

    return $url
}

################################################################################
# UrlLogDownload
#   Starts downloading one of the sites in the log.
#
# Parameters
#   url: The site we are talking about.
################################################################################
proc UrlLogDownload {url} {
    variable urlLog
    global   getleftState

    if {$url==""} return

    PutUrlInQueue $url "$urlLog($url,localDir)" "$urlLog($url,options)"

    if {$getleftState(downloading)==0} {
        AutomaticDownload $url                
    }

    return
}

################################################################################
# UrlLogUpdate
#   Starts updating one of the sites in the log, actually it only makes sure
#   the 'update' option is set before passing it the 'UrlLogDownload'
#
# Parameters
#   url: The site we are talking about.
################################################################################
proc UrlLogUpdate {url} {
    variable urlLog

    if {$url==""} return

    set optionsTmp $urlLog($url,options)

    regsub {update [0-1]} "$urlLog($url,options)" \
           {update 1}       urlLog($url,options)

    UrlLogDownload $url                

    set urlLog($url,options) $optionsTmp

    return
}

################################################################################
# UrlLogQueue
#   Puts the url and its data into Getleft's url queue for future downloading.
#
# Parameters
#   url: The site we are talking about.
################################################################################
proc UrlLogQueue {url} {
    variable urlLog

    if {$url==""} return

    PutUrlInQueue $url "$urlLog($url,localDir)" "$urlLog($url,options)"

    return
}

################################################################################
# UrlLogChangeOptions
#   Changes the options for downloading an URL in the urlList.
#
# Parameters
#   url: The site we are talking about.
################################################################################
proc UrlLogChangeOptions {url} {
    variable urlLog

    if {$url==""} return

    array set ::downOptionsTemp $urlLog($url,options)
    set done [::Opciones::ChooseOptions ::downOptionsTemp]
    if {$done==1} {
        set urlLog($url,options) [array get ::downOptionsTemp]
    }
    return
}

################################################################################
# UrlLogChangeDir
#   Changes the directory to store the download.
#
# Parameters
#   url: The site we are talking about.
################################################################################
proc UrlLogChangeDir {url} {
    variable urlLog
    variable window

    set baseDir [Dialogos::SelectDirectory [file dirname $urlLog($url,localDir)]]
    if {$baseDir!=""} {
        set urlLog($url,localDir) $baseDir
        set index [UrlLogUrlIndex $url]
        $window(listbox) cellconfigure $index,1 -text $baseDir
    }

    return
}

################################################################################
# UrlLogRemove
#    Removes one Url from the log 
#
# Parameters
#    url: The url to remove.
################################################################################
proc UrlLogRemove {url} {
    variable urlLog
    variable window

    set index [lsearch $urlLog(urlList) $url]
    if {$index==-1} {
        return
    }

    set urlLog(urlList) [lreplace $urlLog(urlList) $index $index]
    catch {unset urlLog($url,locaDir)}
    catch {unset urlLog($url,date)}
    catch {unset urlLog($url,options)}
    catch {unset urlLog($url,keep)}

    set index [UrlLogUrlIndex $url]
    if {$index==-1} {
        return
    }
    $window(listbox) delete $index
    ActivateTaskbarButtons

    return
}

################################################################################
# UrlLogRightClick
#   This procedure is invoked when the user right-clicks in the multicolumn
#   list with the sites, it makes a context menu pop up with the
#   appropiate options
#
# Parameters
#   win: window where you right-clicked.
#   y,x: the coordinates where you did it for the tablelist widget.
#   X,Y: and for the pop up menu.
################################################################################
proc UrlLogRightClick {win x y X Y} {
    variable window
    variable menus
    variable urlLog
    global getleftState

    set index [UrlLogGetIndex $win $y]
    if {$index==-1} return

    ActivateTaskbarButtons

    set url [$window(listbox) cellcget $index,0 -text]

    $menus(rightClick) entryconfigure 0 -command "getLog::UrlLogDownload $url"
    $menus(rightClick) entryconfigure 1 -command "getLog::UrlLogUpdate   $url"
    $menus(rightClick) entryconfigure 3 -command "getLog::UrlLogQueue    $url"
    $menus(rightClick) entryconfigure 5 \
            -command "getLog::UrlLogChangeOptions $url"
    $menus(rightClick) entryconfigure 6 \
            -command "getLog::UrlLogChangeDir $url"
    $menus(rightClick) entryconfigure 7 \
            -variable getLog::urlLog($url,keep)
    $menus(rightClick) entryconfigure 9 \
            -command "clipboard clear
                      clipboard append $url"
    $menus(rightClick) entryconfigure 11 \
            -command "getLog::UrlLogRemove $url"

    tk_popup $menus(rightClick) $X $Y

    return
}

################################################################################
# UrlLogDoubleClick
#   This procedure is invoked when the user double-clicks in the multicolumn
#   list with the sites, it makes a context menu pop up with the
#   appropiate options
#
# Parameters
#   win: window where you right-clicked.
#   y,x: the coordinates where you did it for the tablelist widget.
#   X,Y: and for the pop up menu.
################################################################################
proc UrlLogDoubleClick {win x y X Y} {
    variable window
    variable menus
    variable urlLog
    global getleftState

    set index [UrlLogGetIndex $win $y]
    if {$index==-1} return

    ActivateTaskbarButtons

    set url [$window(listbox) cellcget $index,0 -text]

    $menus(doubleClick) entryconfigure 0 -command "getLog::UrlLogDownload $url"
    $menus(doubleClick) entryconfigure 1 -command "getLog::UrlLogUpdate   $url"

    tk_popup $menus(doubleClick) $X $Y

    return
}

###############################################################################
# GetLogSortUrl
#    This procedure is used to sort the list depending on the url.
#
# Parameter:
#    firstUrl, secondUrl: The urls to compare
#
# Returns:
#     '-1', '0' or '1' if the first url is 'smaller' than, equal to or 
#     'greater' than the second.
###############################################################################
proc GetLogSortUrl {firstUrl secondUrl} {
    global labelMonths

    regexp {(?:://)(.*)} $firstUrl  nada firstUrl
    regexp {(?:://)(.*)} $secondUrl nada secondUrl

    set firstUrl  [string tolower $firstUrl]
    set secondUrl [string tolower $secondUrl]

    if {$firstUrl<$secondUrl} {
        return -1
    } elseif {$firstUrl==$secondUrl} {
        return 0
    }
    return 1
}

###############################################################################
# GetLogSortDate
#    This procedure is used to sort the list depending on the downloading date.
#
# Parameter:
#    firstDate, secondDate: The dates to compare
#
# Returns:
#     '-1', '0' or '1' if the first date is earlier than, equal to or later
#     than the second.
###############################################################################
proc GetLogSortDate {firstDate secondDate} {
    global labelMonths

    regexp {(.*)(?:-)(.*)(?:-)(.*)} $firstDate  nada firstDay  firstMonth  firstYear
    regexp {(.*)(?:-)(.*)(?:-)(.*)} $secondDate nada secondDay secondMonth secondYear

    set first  [clock scan $firstDay-$labelMonths($firstMonth)-$firstYear]
    set second [clock scan $secondDay-$labelMonths($secondMonth)-$secondYear]

    if {$first<$second} {
        return -1
    } elseif {$first==$second} {
        return 0
    }
    return 1
}

###############################################################################
# GetLogWindowRow
#    Puts one row in the log window.
###############################################################################
proc GetLogWindowRow {url dir} {
    global labelMonths labelDialogs
    variable window
    variable urlLog

    set month $labelMonths([clock format $urlLog($url,date) -format %B])

    # This is a workaround, in Tcl8.4a3 if I write '-format "%d-$month-%Y"'
    # unicode characters in '$month' get mangled.
    set date0 [clock format $urlLog($url,date) -format "%d"]
    set date1 [clock format $urlLog($url,date) -format "%Y"]
    set date  "$date0-$month-$date1"

    $window(listbox) insert end [list $url $dir $date]

    return
}

###############################################################################
# GetLogWindowPopulate
#    Puts the urls in the log in the listbox widget.
###############################################################################
proc GetLogWindowPopulate {} {
    variable urlLog
    variable window

    foreach url $urlLog(urlList) {
        GetLogWindowRow $url $urlLog($url,localDir)
    }

    return
}

###############################################################################
# GetLogWindow
#    Creates the toplevel window with the log in it.
###############################################################################
proc GetLogWindow {} {
    global dirGetleft labelDialogs labelTitles labelMenus
    variable scrollbars
    variable taskbar
    variable window
    variable menus

    if {[winfo exists .getLogWindow]} {
        focus $window(toplevel)
        return
    }

    set coord(x) [winfo rootx .]
    set coord(y) [winfo rooty .]

    set win [toplevel .getLogWindow]
    wm title $win $labelTitles(getLog)
    wm geometry  $win +[expr {$coord(x)+100}]+[expr {$coord(y)+15}]

    set extFrame [frame $win.extFrame]
    set intFrame [frame $extFrame.intFrame]

    set taskbarFrame [frame $intFrame.taskbarFrame]
    set logFrame     [frame $intFrame.logFrame -bd 2 -relief sunken]

    set scrollBars(y) [scrollbar $logFrame.scrolly -orient vertical   \
            -command  [list $logFrame.listbox yview]]
    set scrollBars(x) [scrollbar $logFrame.scrollx -orient horizontal \
            -command  [list $logFrame.listbox xview]]

    set listbox [::tablelist::tablelist $logFrame.listbox       \
            -bd 0 -height 10 -width 90  -bg white -stretch all  \
            -columns [list 0 $labelDialogs(url)      left       \
                           0 $labelDialogs(localDir) left       \
                           0 $labelDialogs(date)     left]      \
		    -showarrow 1 -labelcommand tablelist::sortByColumn  \
            -xscrollcommand "$scrollBars(x) set"                \
            -yscrollcommand "$scrollBars(y) set"]

    $listbox columnconfigure 0 -sortmode command
    $listbox columnconfigure 0 -sortcommand ::getLog::GetLogSortUrl
    $listbox columnconfigure 1 -sortmode command
    $listbox columnconfigure 1 -sortcommand ::getLog::GetLogSortUrl
    $listbox columnconfigure 2 -sortmode command
    $listbox columnconfigure 2 -sortcommand ::getLog::GetLogSortDate

    set window(toplevel) $win
    set window(listbox)  $listbox

    DefineTaskbarButtons $taskbarFrame

    pack $extFrame -padx 10 -pady 10 -expand true -fill both
    pack $intFrame -expand true -fill both
    pack $taskbarFrame -padx 2  -fill x
    pack $logFrame -expand true -fill both
    grid $scrollBars(y) -in $logFrame -row 0 -column 1 -sticky ns
    grid $scrollBars(x) -in $logFrame -row 1 -column 0 -sticky ew
    grid $listbox       -in $logFrame -row 0 -column 0 -sticky news \
            -padx 0 -pady 0

    grid columnconfigure $logFrame 0 -weight 1
    grid columnconfigure $logFrame 1 -weight 0
    grid rowconfigure    $logFrame 0 -weight 1
    grid rowconfigure    $logFrame 1 -weight 0

    bind [$listbox bodypath] <ButtonRelease-1> {
        getLog::ActivateTaskbarButtons
    }
    bind [$listbox bodypath] <Button-3> {
        getLog::UrlLogRightClick %W %x %y %X %Y
    }
    bind [$listbox bodypath] <Double-Button-1> {
        getLog::UrlLogDoubleClick %W %x %y %X %Y
    }
    bind $win <Escape> "destroy $win"

    focus [$listbox bodypath]

    set menus(rightClick) [menu $listbox.menur -tearoff 0]
    $menus(rightClick) add command -label $labelMenus(download)
    $menus(rightClick) add command -label $labelMenus(update)
    $menus(rightClick) add separator
    $menus(rightClick) add command -label $labelMenus(queue)
    $menus(rightClick) add separator
    $menus(rightClick) add command -label $labelMenus(options)
    $menus(rightClick) add command -label $labelMenus(localDir)
    $menus(rightClick) add check   -label $labelMenus(notErase)
    $menus(rightClick) add separator
    $menus(rightClick) add command -label $labelDialogs(copyLink)
    $menus(rightClick) add separator
    $menus(rightClick) add command -label $labelMenus(delete)

    set menus(doubleClick) [menu $listbox.menud -tearoff 0]
    $menus(doubleClick) add command -label $labelMenus(download)
    $menus(doubleClick) add command -label $labelMenus(update)

    ActivateTaskbarButtons

    return
}

###############################################################################
# GetLog
#    Procedures that controls what happens with the log, it cares care of 
#    reading it into memory, invoking the procedure to draw the window and
#    whatever else needed.
###############################################################################
proc GetLog {} {
    variable urlLog

    if {![info exists urlLog]} {
        UrlLogRead
    }

    GetLogWindow
    GetLogWindowPopulate

    return
}

# When we start Getleft we read the log

UrlLogRead

}


